% 
close all 
clear all

set(0,'defaultTextFontName', 'Arial');
set(0,'defaultTextFontSize', 14);
set(0,'defaultTextFontWeight', 'Bold');
set(0,'defaultTextBackgroundColor', 'none');
set(0,'defaultLineLineWidth', 2);


RandStream.setDefaultStream(RandStream('mt19937ar','seed',0));

doUpdateFlag = true;

%%

dt = .001;
g = 9.81;
b = 0.0025;

R = diag([0.01^2,0.005^2]);
stdR = sqrt(R);
Q = 100^2;
stdQ = sqrt(Q);

trueState = [0;0];

kalman.mu = [0;0];
kalman.P = zeros(2,2);

A = [1, dt; 0, 1-b];
B = [0; dt];
C = [1000, 0];
u = g;

nt = 2001;

history.trueState = nan(nt,2);
history.mu_pred = nan(nt,2);
history.stdDev_pred = nan(nt,2);
history.cov_pred = nan(nt,1);

history.mu_upd = nan(nt,2);
history.stdDev_upd = nan(nt,2);
history.cov_upd = nan(nt,1);

history.trueState(1,:) = trueState';
history.mu_pred(1,:) = kalman.mu';
history.stdDev_pred(1,:) = sqrt(diag(kalman.P))';
history.cov_pred(1,:) = kalman.P(1,2);

history.nominalMeas = nan(nt,1);
history.noisedMeas = nan(nt,1);

history.expectedMeas = nan(nt,1);
history.expectedMeasStd = nan(nt,1);



%%
for t = 2 : nt
    trueState = A * trueState + B*u + stdR * randn(2,1);
    kalman.mu = A * kalman.mu + B*u;
    kalman.P  = A * kalman.P * A' + R;
    
    
    history.trueState(t,:) = trueState';
    history.mu_pred(t,:) = kalman.mu';
    history.stdDev_pred(t,:) = sqrt(diag(kalman.P))';
    history.cov_pred(t,:) = kalman.P(1,2);
    
    history.nominalMeas(t,:) = (C * trueState)';
    history.noisedMeas(t,:) = history.nominalMeas(t,:) + (stdQ * randn(1,1))';

    y = C *  kalman.mu;
    S = C * kalman.P * C' + Q;
    z = history.noisedMeas(t,:)';
    
    history.expectedMeas(t,:) = y';
    history.expectedMeasStd(t,:) = sqrt(diag(S))';
    
    if(doUpdateFlag)
        K = kalman.P * C' * inv(S);
        kalman.mu = kalman.mu + K * (z - y);
        kalman.P = (eye(2) - K*C)*kalman.P;
        history.mu_upd(t,:) = kalman.mu';
        history.stdDev_upd(t,:) = sqrt(diag(kalman.P))';
        history.cov_upd(t,:) = kalman.P(1,2);
    end
    
end

time = dt*[0:nt-1];

figure()
subplot(2,1,1)
title('altitude')
hold on
plot(time,history.trueState(:,1),'g');
plot(time,history.mu_pred(:,1),'b');
plot(time,history.mu_pred(:,1) + 3*history.stdDev_pred(:,1),'b--');
plot(time,history.mu_pred(:,1) - 3*history.stdDev_pred(:,1),'b--');
legend('true','predicted');
hold off
subplot(2,1,2)
title('speed')
hold on
plot(time,history.trueState(:,2),'g');
plot(time,history.mu_pred(:,2),'b');
plot(time,history.mu_pred(:,2) + 3*history.stdDev_pred(:,2),'b--');
plot(time,history.mu_pred(:,2) - 3*history.stdDev_pred(:,2),'b--');
% legend('true','predicted');
hold off

figure()
title('measure')
hold on
plot(time,history.noisedMeas(:,1),'r');
plot(time,history.nominalMeas(:,1),'g');
legend('noised measure','nominal measure');

figure()
title('expected measure')
hold on
plot(time,history.noisedMeas(:,1),'r');
plot(time,history.expectedMeas(:,1),'b');
plot(time,history.expectedMeas(:,1) + 3 * history.expectedMeasStd(:,1),'b--');
plot(time,history.expectedMeas(:,1) - 3 * history.expectedMeasStd(:,1),'b--');
legend('noides measure (real)','expected measure');


figure()
subplot(2,1,1)
title('altitude')
hold on
plot(time,history.trueState(:,1),'g');
plot(time,history.mu_upd(:,1),'m');
plot(time,history.mu_upd(:,1) + 3*history.stdDev_upd(:,1),'m--');
plot(time,history.mu_upd(:,1) - 3*history.stdDev_upd(:,1),'m--');
legend('true','updated');
hold off
subplot(2,1,2)
title('speed')
hold on
plot(time,history.trueState(:,2),'g');
plot(time,history.mu_upd(:,2),'m');
plot(time,history.mu_upd(:,2) + 3*history.stdDev_upd(:,2),'m--');
plot(time,history.mu_upd(:,2) - 3*history.stdDev_upd(:,2),'m--');
% legend('true','updated');
hold off

figure()
subplot(2,1,1)
title('altitude error')
hold on
plot(time,history.trueState(:,1) - history.mu_upd(:,1),'r');
plot(time,3*history.stdDev_upd(:,1),'r--');
plot(time,-3*history.stdDev_upd(:,1),'r--');
hold off
subplot(2,1,2)
title('speed error')
hold on
plot(time,history.trueState(:,2)- history.mu_upd(:,2),'r');
plot(time,3*history.stdDev_upd(:,2),'r--');
plot(time,- 3*history.stdDev_upd(:,2),'r--');
hold off

figure()
subplot(2,1,1)
title('altitude')
hold on
plot(time,history.trueState(:,1),'g');
plot(time,history.mu_pred(:,1),'b');
plot(time,history.mu_pred(:,1) + 3*history.stdDev_pred(:,1),'b--');
plot(time,history.mu_pred(:,1) - 3*history.stdDev_pred(:,1),'b--');
plot(time,history.mu_upd(:,1),'b');
plot(time,history.mu_upd(:,1) + 3*history.stdDev_upd(:,1),'b--');
plot(time,history.mu_upd(:,1) - 3*history.stdDev_upd(:,1),'b--');
legend('true','predicted');
hold off
subplot(2,1,2)
title('speed')
hold on
plot(time,history.trueState(:,2),'g');
plot(time,history.mu_pred(:,2),'b');
plot(time,history.mu_pred(:,2) + 3*history.stdDev_pred(:,2),'b--');
plot(time,history.mu_pred(:,2) - 3*history.stdDev_pred(:,2),'b--');
% legend('true','predicted');
hold off